@@ -58,12 +58,14 @@ class UStr(ustr):
     byte sequences.
 
     """
+
     def __new__(cls, string, encoding):
 
         if isinstance(string, UStr):
             if encoding != string.encoding:
-                raise ValueError('Encoding conflict: %s vs. %s'
-                                 % (string.encoding, encoding))
+                raise ValueError(
+                    'Encoding conflict: %s vs. %s' % (string.encoding, encoding)
+                )
             string = ustr(string)
 
         obj = ustr.__new__(cls, string)
@@ -71,13 +73,23 @@ class UStr(ustr):
         return obj
 
 
+def decode_maybe(value, encoding, errors='strict'):
+    """Decode a value when the "decode" method exists"""
+    if hasattr(value, 'decode'):
+        result = value.decode(encoding, errors=errors)
+    else:
+        result = value
+    return result
+
+
 def decode(value, encoding=None, errors='strict'):
-    """decode(encoded_string) returns an unencoded unicode string
-    """
+    """decode(encoded_string) returns an unencoded unicode string"""
     if value is None:
         result = None
     elif isinstance(value, ustr):
         result = UStr(value, ENCODING)
+    elif encoding == 'bytes':
+        result = value
     else:
         result = None
         if encoding is None:
@@ -101,8 +113,7 @@ def decode(value, encoding=None, errors='strict'):
 
 
 def encode(string, encoding=None):
-    """encode(unencoded_string) returns a string encoded in utf-8
-    """
+    """encode(unencoded_string) returns a string encoded in utf-8"""
     if not isinstance(string, ustr):
         return string
     return string.encode(encoding or ENCODING, 'replace')
@@ -156,13 +167,17 @@ def readline(fh, encoding=None):
 
 
 @interruptable
-def start_command(cmd, cwd=None, add_env=None,
-                  universal_newlines=False,
-                  stdin=subprocess.PIPE,
-                  stdout=subprocess.PIPE,
-                  no_win32_startupinfo=False,
-                  stderr=subprocess.PIPE,
-                  **extra):
+def start_command(
+    cmd,
+    cwd=None,
+    add_env=None,
+    universal_newlines=False,
+    stdin=subprocess.PIPE,
+    stdout=subprocess.PIPE,
+    no_win32_startupinfo=False,
+    stderr=subprocess.PIPE,
+    **extra
+):
     """Start the given command, and return a subprocess object.
 
     This provides a simpler interface to the subprocess module.
@@ -208,9 +223,20 @@ def start_command(cmd, cwd=None, add_env=None,
             CREATE_NO_WINDOW = 0x08000000
             extra['creationflags'] = CREATE_NO_WINDOW
 
-    return subprocess.Popen(cmd, bufsize=1, stdin=stdin, stdout=stdout,
-                            stderr=stderr, cwd=cwd, env=env,
-                            universal_newlines=universal_newlines, **extra)
+    # Use line buffering when in text/universal_newlines mode,
+    # otherwise use the system default buffer size.
+    bufsize = 1 if universal_newlines else -1
+    return subprocess.Popen(
+        cmd,
+        bufsize=bufsize,
+        stdin=stdin,
+        stdout=stdout,
+        stderr=stderr,
+        cwd=cwd,
+        env=env,
+        universal_newlines=universal_newlines,
+        **extra
+    )
 
 
 def prep_for_subprocess(cmd, shell=False):
@@ -248,9 +274,7 @@ def run_command(cmd, *args, **kwargs):
     output = decode(output, encoding=encoding)
     errors = decode(errors, encoding=encoding)
     exit_code = process.returncode
-    return (exit_code,
-            output or UStr('', ENCODING),
-            errors or UStr('', ENCODING))
+    return (exit_code, output or UStr('', ENCODING), errors or UStr('', ENCODING))
 
 
 @interruptable
@@ -277,8 +301,9 @@ def _fork_win32(args, cwd=None, shell=False):
         argv = [encode(arg) for arg in args]
 
     DETACHED_PROCESS = 0x00000008  # Amazing!
-    return subprocess.Popen(argv, cwd=cwd, creationflags=DETACHED_PROCESS,
-                            shell=shell).pid
+    return subprocess.Popen(
+        argv, cwd=cwd, creationflags=DETACHED_PROCESS, shell=shell
+    ).pid
 
 
 def _win32_find_exe(exe):
@@ -298,8 +323,7 @@ def _win32_find_exe(exe):
     # extensions specified in PATHEXT
     if '.' not in exe:
         extensions = getenv('PATHEXT', '').split(os.pathsep)
-        candidates.extend([(exe + ext) for ext in extensions
-                           if ext.startswith('.')])
+        candidates.extend([(exe + ext) for ext in extensions if ext.startswith('.')])
     # search the current directory first
     for candidate in candidates:
         if exists(candidate):
@@ -342,9 +366,11 @@ def wrap(action, fn, decorator=None):
 
 def decorate(decorator, fn):
     """Decorate the result of `fn` with `action`"""
+
     @functools.wraps(fn)
     def decorated(*args, **kwargs):
         return decorator(fn(*args, **kwargs))
+
     return decorated
 
 
@@ -359,7 +385,7 @@ def guess_mimetype(filename):
         mimetype = mimetypes.guess_type(filename)[0]
     except UnicodeEncodeError:
         mimetype = mimetypes.guess_type(encode(filename))[0]
-    except TypeError:
+    except (TypeError, ValueError):
         mimetype = mimetypes.guess_type(decode(filename))[0]
     return mimetype
 
