@@ -475,7 +475,8 @@ def set_clipboard(text):
         return
     clipboard = QtWidgets.QApplication.clipboard()
     clipboard.setText(text, QtGui.QClipboard.Clipboard)
-    clipboard.setText(text, QtGui.QClipboard.Selection)
+    if not utils.is_darwin() and not utils.is_win32():
+        clipboard.setText(text, QtGui.QClipboard.Selection)
     persist_clipboard()
 
 
@@ -558,9 +559,8 @@ def create_treeitem(filename, staged=False, deleted=False, untracked=False):
 
     """
     icon_name = icons.status(filename, deleted, staged, untracked)
-    return TreeWidgetItem(
-        filename, icons.name_from_basename(icon_name), deleted=deleted
-    )
+    icon = icons.name_from_basename(icon_name)
+    return TreeWidgetItem(filename, icon, deleted=deleted)
 
 
 def add_close_action(widget):
@@ -843,6 +843,8 @@ class DebouncingMenu(QtWidgets.QMenu):
     def __init__(self, title, parent):
         QtWidgets.QMenu.__init__(self, title, parent)
         self.created_at = utils.epoch_millis()
+        if hasattr(self, 'setToolTipsVisible'):
+            self.setToolTipsVisible(True)
 
     def mouseReleaseEvent(self, event):
         threshold = DebouncingMenu.threshold_ms
@@ -853,7 +855,10 @@ class DebouncingMenu(QtWidgets.QMenu):
 def add_menu(title, parent):
     """Create a menu and set its title."""
     menu = create_menu(title, parent)
-    parent.addAction(menu.menuAction())
+    if hasattr(parent, 'addMenu'):
+        parent.addMenu(menu)
+    else:
+        parent.addAction(menu.menuAction())
     return menu
 
 
@@ -861,7 +866,7 @@ def create_toolbutton(text=None, layout=None, tooltip=None, icon=None):
     button = tool_button()
     if icon is not None:
         button.setIcon(icon)
-        button.setIconSize(QtCore.QSize(defs.small_icon, defs.small_icon))
+        button.setIconSize(QtCore.QSize(defs.default_icon, defs.default_icon))
     if text is not None:
         button.setText(' ' + text)
         button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
