@@ -11,6 +11,7 @@ if __name__ == '__main__':
 from .git import STDOUT  # noqa
 from .decorators import memoize  # noqa
 from ._version import VERSION  # noqa
+
 try:
     from ._build_version import BUILD_VERSION
 except ImportError:
@@ -23,16 +24,26 @@ _versions = {
     # git difftool moved out of contrib in git 1.6.3
     'git': '1.6.3',
     'python': '2.6',
+    # new: git cat-file --filters --path=<path> SHA1
+    # old: git cat-file --filters blob SHA1:<path>
+    'cat-file-filters-path': '2.11.0',
     # git diff --submodule was introduced in 1.6.6
     'diff-submodule': '1.6.6',
     # git check-ignore was introduced in 1.8.2, but did not follow the same
     # rules as git add and git status until 1.8.5
     'check-ignore': '1.8.5',
+    # git push --force-with-lease
+    'force-with-lease': '1.8.5',
+    # git submodule update --recursive was introduced in 1.6.5
+    'submodule-update-recursive': '1.6.5',
+    # git include.path pseudo-variable was introduced in 1.7.10
+    'config-includes': '1.7.10',
     # git for-each-ref --sort=version:refname
     'version-sort': '2.7.0',
-    # new: git cat-file --filters --path=<path> SHA1
-    # old: git cat-file --filters blob SHA1:<path>
-    'cat-file-filters-path': '2.11.0',
+    # Qt support for QT_AUTO_SCREEN_SCALE_FACTOR and QT_SCALE_FACTOR
+    'qt-hidpi-scale': '5.6.0',
+    # git rev-parse --show-superproject-working-tree was added in 2.13.0
+    'show-superproject-working-tree': '2.13.0',
 }
 
 
@@ -53,8 +64,7 @@ def build_version():
 
 @memoize
 def check_version(min_ver, ver):
-    """Check whether ver is greater or equal to min_ver
-    """
+    """Check whether ver is greater or equal to min_ver"""
     min_ver_list = version_to_list(min_ver)
     ver_list = version_to_list(ver)
     return min_ver_list <= ver_list
@@ -72,8 +82,7 @@ def check_git(context, key):
 
 
 def version_to_list(value):
-    """Convert a version string to a list of numbers or strings
-    """
+    """Convert a version string to a list of numbers or strings"""
     ver_list = []
     for p in value.split('.'):
         try:
