@@ -55,8 +55,7 @@ class LineEdit(QtWidgets.QLineEdit):
 
 
 class LineEditCursorPosition(object):
-    """Translate cursorPositionChanged(int,int) into cursorPosition(int,int)
-    """
+    """Translate cursorPositionChanged(int,int) into cursorPosition(int,int)"""
 
     def __init__(self, widget, row):
         self._widget = widget
@@ -89,7 +88,7 @@ class BaseTextEditExtension(QtCore.QObject):
 
     def _init_flags(self):
         widget = self.widget
-        widget.setMinimumSize(QtCore.QSize(1, 1))
+        widget.setMinimumSize(QtCore.QSize(10, 10))
         widget.setWordWrapMode(QtGui.QTextOption.WordWrap)
         widget.setLineWrapMode(widget.NoWrap)
         if self._readonly:
@@ -591,6 +590,15 @@ class HintedTextEdit(TextEdit):
             self.hint.enable(True)
 
 
+def anchor_mode(select):
+    """Return the QTextCursor mode to keep/discard the cursor selection"""
+    if select:
+        mode = QtGui.QTextCursor.KeepAnchor
+    else:
+        mode = QtGui.QTextCursor.MoveAnchor
+    return mode
+
+
 # The vim-like read-only text view
 
 
@@ -599,27 +607,36 @@ class VimMixin(object):
         self.widget = widget
         self.Base = widget.Base
         # Common vim/unix-ish keyboard actions
+        self.add_navigation('End', hotkeys.GOTO_END)
         self.add_navigation('Up', hotkeys.MOVE_UP, shift=hotkeys.MOVE_UP_SHIFT)
         self.add_navigation('Down', hotkeys.MOVE_DOWN, shift=hotkeys.MOVE_DOWN_SHIFT)
         self.add_navigation('Left', hotkeys.MOVE_LEFT, shift=hotkeys.MOVE_LEFT_SHIFT)
         self.add_navigation('Right', hotkeys.MOVE_RIGHT, shift=hotkeys.MOVE_RIGHT_SHIFT)
         self.add_navigation('WordLeft', hotkeys.WORD_LEFT)
         self.add_navigation('WordRight', hotkeys.WORD_RIGHT)
+        self.add_navigation('Start', hotkeys.GOTO_START)
         self.add_navigation('StartOfLine', hotkeys.START_OF_LINE)
         self.add_navigation('EndOfLine', hotkeys.END_OF_LINE)
 
         qtutils.add_action(
-            widget,
-            'PageUp',
-            lambda: widget.page(-widget.height() // 2),
-            hotkeys.SECONDARY_ACTION,
+            widget, 'PageUp', widget.page_up, hotkeys.SECONDARY_ACTION, hotkeys.UP
+        )
+        qtutils.add_action(
+            widget, 'PageDown', widget.page_down, hotkeys.PRIMARY_ACTION, hotkeys.DOWN
         )
-
         qtutils.add_action(
             widget,
-            'PageDown',
-            lambda: widget.page(widget.height() // 2),
-            hotkeys.PRIMARY_ACTION,
+            'SelectPageUp',
+            lambda: widget.page_up(select=True),
+            hotkeys.SELECT_BACK,
+            hotkeys.SELECT_UP,
+        )
+        qtutils.add_action(
+            widget,
+            'SelectPageDown',
+            lambda: widget.page_down(select=True),
+            hotkeys.SELECT_FORWARD,
+            hotkeys.SELECT_DOWN,
         )
 
     def add_navigation(self, name, hotkey, shift=None):
@@ -629,27 +646,37 @@ class VimMixin(object):
         qtutils.add_action(widget, name, lambda: self.move(direction), hotkey)
         if shift:
             qtutils.add_action(
-                widget, 'Shift' + name, lambda: self.move(direction, True), shift
+                widget, 'Shift' + name, lambda: self.move(direction, select=True), shift
             )
 
     def move(self, direction, select=False, n=1):
         widget = self.widget
         cursor = widget.textCursor()
-        if select:
-            mode = QtGui.QTextCursor.KeepAnchor
-        else:
-            mode = QtGui.QTextCursor.MoveAnchor
-        if cursor.movePosition(direction, mode, n):
-            self.set_text_cursor(cursor)
+        mode = anchor_mode(select)
+        for _ in range(n):
+            if cursor.movePosition(direction, mode, 1):
+                self.set_text_cursor(cursor)
 
-    def page(self, offset):
+    def page(self, offset, select=False):
         widget = self.widget
         rect = widget.cursorRect()
         x = rect.x()
         y = rect.y() + offset
         new_cursor = widget.cursorForPosition(QtCore.QPoint(x, y))
         if new_cursor is not None:
-            self.set_text_cursor(new_cursor)
+            cursor = widget.textCursor()
+            mode = anchor_mode(select)
+            cursor.setPosition(new_cursor.position(), mode)
+
+            self.set_text_cursor(cursor)
+
+    def page_down(self, select=False):
+        widget = self.widget
+        widget.page(widget.height() // 2, select=select)
+
+    def page_up(self, select=False):
+        widget = self.widget
+        widget.page(-widget.height() // 2, select=select)
 
     def set_text_cursor(self, cursor):
         widget = self.widget
@@ -681,10 +708,8 @@ class VimMixin(object):
                 # The cursor is in the middle of the first line of text.
                 # We can't go up ~ jump to the beginning of the line.
                 # Select the text if shift is pressed.
-                if event.modifiers() & Qt.ShiftModifier:
-                    mode = QtGui.QTextCursor.KeepAnchor
-                else:
-                    mode = QtGui.QTextCursor.MoveAnchor
+                select = event.modifiers() & Qt.ShiftModifier
+                mode = anchor_mode(select)
                 cursor.movePosition(QtGui.QTextCursor.StartOfLine, mode)
                 widget.setTextCursor(cursor)
 
@@ -709,8 +734,14 @@ class VimHintedPlainTextEdit(HintedPlainTextEdit):
     def move(self, direction, select=False, n=1):
         return self._mixin.page(direction, select=select, n=n)
 
-    def page(self, offset):
-        return self._mixin.page(offset)
+    def page(self, offset, select=False):
+        return self._mixin.page(offset, select=select)
+
+    def page_up(self, select=False):
+        return self._mixin.page_up(select=select)
+
+    def page_down(self, select=False):
+        return self._mixin.page_down(select=select)
 
     def keyPressEvent(self, event):
         return self._mixin.keyPressEvent(event)
@@ -734,23 +765,40 @@ class VimTextEdit(MonoTextEdit):
     def move(self, direction, select=False, n=1):
         return self._mixin.page(direction, select=select, n=n)
 
-    def page(self, offset):
-        return self._mixin.page(offset)
+    def page(self, offset, select=False):
+        return self._mixin.page(offset, select=select)
+
+    def page_up(self, select=False):
+        return self._mixin.page_up(select=select)
+
+    def page_down(self, select=False):
+        return self._mixin.page_down(select=select)
 
     def keyPressEvent(self, event):
         return self._mixin.keyPressEvent(event)
 
 
-class HintedLineEdit(LineEdit):
-    def __init__(self, context, hint, parent=None):
+class HintedDefaultLineEdit(LineEdit):
+    """A line edit with hint text"""
+
+    def __init__(self, hint, tooltip=None, parent=None):
         LineEdit.__init__(self, parent=parent, get_value=get_value_hinted)
+        if tooltip:
+            self.setToolTip(tooltip)
         self.hint = HintWidget(self, hint)
         self.hint.init()
-        self.setFont(qtutils.diff_font(context))
         # pylint: disable=no-member
         self.textChanged.connect(lambda text: self.hint.refresh())
 
 
+class HintedLineEdit(HintedDefaultLineEdit):
+    """A monospace line edit with hint text"""
+
+    def __init__(self, context, hint, tooltip=None, parent=None):
+        super(HintedLineEdit, self).__init__(hint, tooltip=tooltip, parent=parent)
+        self.setFont(qtutils.diff_font(context))
+
+
 def text_dialog(context, text, title):
     """Show a wall of text in a dialog"""
     parent = qtutils.active_window()
